import logging
import time
from typing import Dict
from azure.cosmosdb.table.tableservice import TableService

GLOBAL_SIM_TABLE="GlobalSimTable"

def get_provider_sim_list(data_usage_provider: str = "", connection_string: str = None):
    '''
    Get the sim list based on provider given.
    If table does not exist, None will be returned
    '''
    try:

        table_service = TableService(connection_string=connection_string)
        sim_entities = table_service.query_entities( GLOBAL_SIM_TABLE, filter=f"PartitionKey eq '{data_usage_provider}'", num_results=1)
        sim_queries = sim_entities.items

        while sim_entities.next_marker:
          next_sim_entities =  table_service.query_entities( GLOBAL_SIM_TABLE, filter=f"PartitionKey eq '{data_usage_provider}'", marker=sim_entities.next_marker)
          sim_queries = [*sim_queries, *next_sim_entities]
          if not next_sim_entities.next_marker:
            break    

        return sim_queries

    except Exception as err:
        logging.error(f'get sim list from {connection_string}: {err}')
        return None

def store_usage_for_provider(data_usage_provider: str , connection_string: str , usage_data: Dict ) -> None:
    '''
    Store the usage data to data_usage_provider table
    The table name generated will have no dash and underscore
    '''
    try:
        usage_data.update( {'PartitionKey':data_usage_provider, 'RowKey': str(time.time())})
        table_service = TableService(connection_string=connection_string)
                
        provider_table = data_usage_provider.replace("_","").replace("-","")

        if not table_service.exists(provider_table):
            table_service.create_table(provider_table)

        table_service.insert_entity(provider_table,entity=usage_data)
 
    except:
        logging.error(f'function: store_usage_for_provider')
        pass

def store_usage_for_provider(data_usage_provider: str , connection_string: str , usage_data: Dict ) -> None:
    '''
    Store the usage data to data_usage_provider table
    The table name generated will have no dash and underscore
    '''
    try:
        usage_data.update( {'PartitionKey':data_usage_provider, 'RowKey': str(time.time())})
        table_service = TableService(connection_string=connection_string)
                
        provider_table = data_usage_provider.replace("_","").replace("-","")

        if not table_service.exists(provider_table):
            table_service.create_table(provider_table)

        table_service.insert_entity(provider_table,entity=usage_data)
 
    except:
        logging.error(f'function: store_usage_for_provider')
        pass




# In object_factory.py
import logging

class ObjectFactory:
    def __init__(self):
        self._builders = {}

    def register_builder(self, key:str, builder):
        '''
        key name must be alphanumeric only
        '''
        if key.isalnum == False:
            raise ValueError("key provided is not alphanumeric")
        self._builders[key] = builder

    def create(self, key, **kwargs):
        try:
            logging.debug(f'creating service builder for {key}')       
            builder = self._builders.get(key)
            if not builder:
                raise ValueError(f'{key} for builder not found in factory')
            return builder(**kwargs)
        except Exception as err:
                logging.error(err)
                raise Exception()

       



import logging
import os
from azure.keyvault.secrets import SecretClient
from azure.identity import DefaultAzureCredential


class ApiSecrets:

    def __init__(self, service_id: str):
        '''
        Retrieves from Azure Key Vault {service_id}-api-key, {service_id}-api-user', {service_id}-api-user' \n
        Requires App Setting, `AZURE_CLIENT_ID`, `AZURE_TENANT_ID`, `AZURE_CLIENT_SECRET`, KEY_VAULT_NAME \n
        Use the Azure CLI to set store the secrets into the Azure Key Vault \n
        '''
        self.key = self._get(f'{service_id}-api-key')
        self.user = self._get(f'{service_id}-api-user')
        self.pwd = self._get(f'{service_id}-api-pwd')

    def _get(self, secret_name):
        try:
            keyVaultName = os.environ["AZURE_KEYVAULT"]
            KVUri = f"https://{keyVaultName}.vault.azure.net"
            credential = DefaultAzureCredential() 
            client = SecretClient(vault_url=KVUri, credential=credential)
            logging.debug(f'retrieving {secret_name}')
            return client.get_secret(secret_name).value

        except Exception as err:
            logging.warn(f'{err} - {secret_name} unretrievable from Keyvault, {keyVaultName}')
            return ''



from .bw_object_factory import ObjectFactory
from .bw_api_service_SGSingtelM2M import SGSingtelM2MServiceBuilder
from .bw_api_service_kpn import KPNServiceBuilder


class DataUsageApiServiceProvider(ObjectFactory):
    def get(self, service_id: str):
        '''
        get the API service provider for [service_id]   
        '''
        return self.create(service_id)

class DataUsageProviders():
    '''
    A Provider for all registered API services
    '''
    def __init__(self) -> None:
        self.services = DataUsageApiServiceProvider()
        self.services.register_builder('SG-Singtel-M2M', SGSingtelM2MServiceBuilder())
        self.services.register_builder('KPN', KPNServiceBuilder())



import requests
from .bw_api_secrets import ApiSecrets

class SGSingtelM2MService:
    base_url = 'https://restapi1.jasper.com/rws/api/v1/devices'

    def __init__(self):
        self._api_secrets = ApiSecrets('SG-Singtel-M2M')

    def request_usage(self, iccid):
        try:
            response = requests.get(self.base_url + '/'+iccid+'/ctdUsages', auth=(self._api_secrets.user, self._api_secrets.key))
            response.raise_for_status()
            return response.json()
        except Exception :
            raise


class SGSingtelM2MServiceBuilder():
    def __init__(self):
        self._instance = None

    def __call__(self, **_ignored):
        if not self._instance:
            self._instance = SGSingtelM2MService()
        return self._instance





import os

def get_storage_connection_string(production_storage="AzureWebJobsStorage"):
    '''
    Get the storage connection string based on environment.
    The "AzureWebJobsStorage" in local.settings.json must be set to "UseDevelopmentStorage=true"
    The Azurite emulator storage connection string is to be set in the variable "EmulatorStorage".
    '''
    connection_string = os.getenv(production_storage)
    if connection_string == "UseDevelopmentStorage=true":
        connection_string = os.getenv("EmulatorStorage")
    return connection_string



import logging
from azure.storage.queue import QueueClient


def get_data_usage_provider_queue(data_usage_provider: str = "", connection_string: str = None):
    '''
    Get the queue client based on data_usage_provider given.
    If queue does not exist, a new one will be created with the
    queue storage name from sim provider name and
    transformed into lowercase as required
    '''
    try:
        queue = QueueClient.from_connection_string(
            conn_str=connection_string, queue_name=data_usage_provider.lower())
        try:
            queue.create_queue()
        except Exception as err:
            logging.warn(f'function: get_data_usage_provider_queue -> {err}')
            pass
        return queue
    except :
        logging.error(f'function: get_data_usage_provider_queue -> QueueClient')
        raise



def get_query_message_list(queue: QueueClient, query_set_size=int):
    '''
    Read the messages in the queue given and return the set of messages 
    as per size given. The returned message will be removed from the queue.
    '''
    queue_message_pages = list(queue.receive_messages(
        messages_per_page=query_set_size).by_page())
    queue_message_list = []

    if len(queue_message_pages) == 0:
        return [], True

    for queue_message in queue_message_pages[0]:
        queue_message_list.append(queue_message)
        queue.delete_message(queue_message)

    is_queue_size_low = False
    if len(queue_message_pages) < 1:
        is_queue_size_low = True

    return queue_message_list, is_queue_size_low
